use super::User;
use crate::model::user::LoginError;
use crate::{
    model::{mail::Mail, notification::Notification, role::Role},
    special_user, SCHECKBUCH,
};
use rocket::async_trait;
use rocket::http::Status;
use rocket::request;
use rocket::request::FromRequest;
use rocket::request::Outcome;
use rocket::Request;
use sqlx::SqlitePool;
use std::ops::Deref;

special_user!(ScheckbuchUser, +"scheckbuch");

impl ScheckbuchUser {
    async fn from(user: User, db: &SqlitePool, mail: &str, smtp_pw: &str) -> Result<(), String> {
        // TODO: see when/how to invoke this function (explicit `Neue Person hinzufügen` button?
        // Button to transition existing users to scheckbuch? Automatically called when
        // `scheckbuch` is newly selected as role?
        if user.has_role(db, "scheckbuch").await {
            return Err("User is already a scheckbuch".into());
        }

        // TODO: do we allow e.g. DonauLinz to scheckbuch?

        let scheckbuch = Role::find_by_name(db, "scheckbuch").await.unwrap();
        user.add_role(db, &scheckbuch).await.unwrap();

        // TODO: remove all other `membership_type` roles

        let new_user = Self::new(db, user).await.unwrap();

        new_user
            .send_welcome_mail_to_user(db, mail, smtp_pw)
            .await?;
        new_user.notify_coxes_about_new_scheckbuch(db).await;

        Ok(())
    }
    async fn send_welcome_mail_to_user(
        &self,
        db: &SqlitePool,
        mail: &str,
        smtp_pw: &str,
    ) -> Result<(), String> {
        Mail::send_single(
            db,
            mail,
            "ASKÖ Ruderverein Donau Linz | Dein Scheckbuch wartet auf Dich",
            format!(
"Hallo {0},

herzlich willkommen beim ASKÖ Ruderverein Donau Linz! Wir freuen uns sehr, dass Du Dich entschieden hast, das Rudern bei uns auszuprobieren. Mit Deinem Scheckbuch kannst Du jetzt an fünf Ausfahrten teilnehmen und so diesen Sport in seiner vollen Vielfalt erleben. Falls du die {1} € noch nicht bezahlt hast, nimm diese bitte zur nächsten Ausfahrt mit (oder überweise sie auf unser Bankkonto [dieses findest du auf https://rudernlinz.at]).

Für die Organisation unserer Ausfahrten nutzen wir app.rudernlinz.at. Logge Dich bitte mit Deinem Namen ('{0}', ohne Anführungszeichen) ein. Beim ersten Mal kannst Du das Passwortfeld leer lassen. Unter 'Geplante Ausfahrten' kannst Du Dich jederzeit für eine Ausfahrt anmelden. Wir bieten mindestens einmal pro Woche Ausfahrten an, sowohl für Anfänger als auch für Fortgeschrittene (A+F Rudern). Zusätzliche Ausfahrten werden von unseren Steuerleuten ausgeschrieben, öfters reinschauen kann sich also lohnen :-)

Nach deinen 5 Ausfahrten würden wir uns freuen, dich als Mitglied in unserem Verein begrüßen zu dürfen.

Wir freuen uns darauf, Dich bald am Wasser zu sehen und gemeinsam tolle Erfahrungen zu sammeln!

Riemen- & Dollenbruch,
ASKÖ Ruderverein Donau Linz", self.name, SCHECKBUCH/100),
            smtp_pw,
        ).await?;

        Ok(())
    }

    async fn notify_coxes_about_new_scheckbuch(&self, db: &SqlitePool) {
        Notification::create_for_steering_people(
            db,
            &format!(
                "Liebe Steuerberechtigte, {} hat nun ein Scheckbuch. Wie immer, freuen wir uns wenn du uns beim A+F Rudern unterstützt oder selber Ausfahrten ausschreibst. Bitte beachte, dass Scheckbuch-Personen nur Ausfahrten sehen, bei denen 'Scheckbuch-Anmeldungen erlauben' ausgewählt wurde.",
                self.name
            ),
            "Neues Scheckbuch",
            None,None
        )
        .await;
    }
}
